LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_signed.ALL;
USE ieee.numeric_std.ALL;
USE work.used_functions_pkg.ALL;

PACKAGE PFB_PKG IS

  -- NOTES and PACKAGES INSTRUCTIONS :
  -- please cf. sub_packages instructions
  --
  -- this block takes real and imaginary part every other sample, for a total
  -- of 2*cst_nb_samples_adc_in_pfb samples in input.
  -- does not work yet for 20 subfilters (fir to be re-tested?)

  -- input
  CONSTANT cst_w_in_pfb              : natural := 6;
  CONSTANT cst_w_out_pfb             : natural := 6;
  CONSTANT cst_nb_samples_adc_in_pfb : natural := 80;  -- a complete sample is a real AND an imag (both following)

  -- polyphase filter
  CONSTANT cst_w_coeffs_polyfir_pfb            : natural := 8;  -- polyfir coeffs bitwidth
  CONSTANT cst_nb_coeffs_polyfir_pfb           : natural := 200;
  CONSTANT cst_polyfir_downsampling_factor_pfb : natural := 8;

  -- dft
  CONSTANT cst_nb_subfilters_pfb                : natural := 10;
  CONSTANT cst_w_precision_winograd5_coeffs_pfb : natural := 8;  -- dft winograd coeffs bitwidth
  CONSTANT cst_w_precision_radix2_coeffs_pfb    : natural := 8;  -- dft butterfly coeffs bitwidth

  CONSTANT cst_nb_bits_shift_round_pfb : natural := 19;  -- the number of MSBs to avoid (sign bits) before rounding. For cst_nb_subfilters_pfb=10, 19 works well; for cst_nb_subfilters_pfb=20, 23 should work well. See bellow for a potential correct formula, but please test for your application, don't trust this formula.

  -- calculations --
  CONSTANT cst_log2_sup_nb_coeffs_subfilter_pfb : natural := log2_sup_integer(number => cst_nb_coeffs_polyfir_pfb/cst_nb_subfilters_pfb);
  CONSTANT cst_w_polyfir_out_dft_in_pfb         : natural := cst_w_in_pfb + cst_w_coeffs_polyfir_pfb+cst_log2_sup_nb_coeffs_subfilter_pfb;
  CONSTANT cst_log2_nb_parallel_winograd_pfb    : natural := log2_sup_integer(number => cst_nb_subfilters_pfb/5);
  CONSTANT cst_nb_parallel_firs_dfts_pfb        : natural := cst_nb_samples_adc_in_pfb/cst_polyfir_downsampling_factor_pfb;

  --CONSTANT cst_nb_bits_shift_round_pfb : natural := 15+4*cst_log2_nb_parallel_winograd_pfb; -- potential corect formula

  -- TYPES
  SUBTYPE smpl_real_imag_adc_data_in_pfb IS std_logic_vector(cst_w_in_pfb-1 DOWNTO 0);
  SUBTYPE smpl_real_imag_polyfir_out_dft_in_pfb IS std_logic_vector(cst_w_polyfir_out_dft_in_pfb-1 DOWNTO 0);
  SUBTYPE smpl_real_imag_dft_out_pfb IS std_logic_vector(cst_w_out_pfb-1 DOWNTO 0);
  --TYPE vect_fir_out_dft_in_pfb IS ARRAY (0 TO cst_nb_subfilters_pfb-1) OF smpl_real_imag_polyfir_out_dft_in_pfb;
  TYPE vect_adc_data_out_pfb IS ARRAY (0 TO 2*cst_nb_samples_adc_in_pfb-1) OF smpl_real_imag_adc_data_in_pfb;

  TYPE vect_dft_output_pfb IS ARRAY (0 TO 2*cst_nb_parallel_firs_dfts_pfb*cst_nb_subfilters_pfb-1) OF smpl_real_imag_dft_out_pfb;


END PACKAGE PFB_PKG;

